/*
 * Copyright (c) 2006-2007 Extreme! Lab, Indiana University. All rights reserved.
 *
 * This software is open source. See the bottom of this file for the license.
 *
 * $Id: WSDLUtil.java,v 1.12 2007/02/08 22:20:42 sshirasu Exp $
 */
package edu.indiana.extreme.xbaya.util;

import java.net.URI;

import javax.xml.namespace.QName;

import org.xmlpull.infoset.XmlElement;

import xsul5.MLogger;
import xsul5.wsdl.WsdlBinding;
import xsul5.wsdl.WsdlDefinitions;
import xsul5.wsdl.WsdlPortType;
import xsul5.wsdl.WsdlPortTypeOperation;
import xsul5.wsdl.WsdlUtil;
import edu.indiana.extreme.xbaya.component.ComponentException;

/**
 * @author Satoshi Shirasuna
 */
public class WSDLUtil {

    private static final MLogger logger = MLogger.getLogger();

    /**
     * @param wsdlString
     * @return The WSDL
     * @throws ComponentException
     */
    public static WsdlDefinitions stringToWSDL(String wsdlString)
            throws ComponentException {
        try {
            XmlElement wsdlElement = XMLUtil.stringToXmlElement(wsdlString);
            WsdlDefinitions definitions = new WsdlDefinitions(wsdlElement);
            return definitions;
        } catch (RuntimeException e) {
            throw new ComponentException(e);
        }
    }

    /**
     * @param definitions3
     * @return The WsdlDefinitions (XSUL5)
     */
    public static xsul5.wsdl.WsdlDefinitions wsdlDefinitions3ToWsdlDefintions5(
            xsul.wsdl.WsdlDefinitions definitions3) {

        return new xsul5.wsdl.WsdlDefinitions(XMLUtil
                .xmlElement3ToXmlElement5(definitions3));
    }

    /**
     * @param definitions5
     * @return The WsdlDefinitions (XSUL3)
     */
    public static xsul.wsdl.WsdlDefinitions wsdlDefinitions5ToWsdlDefintions3(
            xsul5.wsdl.WsdlDefinitions definitions5) {

        return new xsul.wsdl.WsdlDefinitions(XMLUtil
                .xmlElement5ToXmlElement3(definitions5.xml()));
    }

    /**
     * @param definitions
     * @return The name of the WSDL.
     */
    public static String getWSDLName(WsdlDefinitions definitions) {
        String wsdlName = definitions.xml().attributeValue(
                WSConstants.NAME_ATTRIBUTE);
        if (wsdlName == null) {
            // name is optional.
            wsdlName = "";
        }
        return wsdlName;
    }

    /**
     * @param definitions
     * @return The QName of the WSDL.
     */
    public static QName getWSDLQName(WsdlDefinitions definitions) {
        String targetNamespace = definitions.getTargetNamespace();
        String wsdlName = getWSDLName(definitions);
        return new QName(targetNamespace, wsdlName);
    }

    /**
     * @param definitions
     * @return The first portType
     * @throws ComponentException
     */
    public static WsdlPortType getFirstPortType(WsdlDefinitions definitions)
            throws ComponentException {
        for (WsdlPortType portType : definitions.portTypes()) {
            return portType;
        }
        throw new ComponentException("No portType is defined in WSDL");
    }

    /**
     * @param definitions
     * @return The QName of the first portType.
     * @throws ComponentException
     */
    public static QName getFirstPortTypeQName(WsdlDefinitions definitions)
            throws ComponentException {
        String targetNamespace = definitions.getTargetNamespace();
        for (WsdlPortType portType : definitions.portTypes()) {
            String portTypeName = portType.getName();
            QName portTypeQName = new QName(targetNamespace, portTypeName);
            return portTypeQName;
        }
        throw new ComponentException("No portType is defined.");
    }

    /**
     * @param definitions
     * @param portTypeQName
     * @return The name of the first operation in a given portType.
     * @throws ComponentException
     */
    public static String getFirstOperationName(WsdlDefinitions definitions,
            QName portTypeQName) throws ComponentException {
        WsdlPortType portType = definitions.getPortType(portTypeQName
                .getLocalPart());
        for (WsdlPortTypeOperation operation : portType.operations()) {
            String operationName = operation.getOperationName();

            // XXX Temporary solution to skip some GFac specific operations.
            if ("Shutdown".equals(operationName)) {
                continue;
            } else if ("Kill".equals(operationName)) {
                continue;
            } else if ("Ping".equals(operationName)) {
                continue;
            }

            return operationName;
        }
        throw new ComponentException("No operation is defined");
    }

    /**
     * @param definitions
     * @return The cloned WsdlDefinitions
     */
    public static WsdlDefinitions deepClone(WsdlDefinitions definitions) {
        return new WsdlDefinitions(XMLUtil.deepClone(definitions.xml()));
    }

    /**
     * @param definitions
     * @param paramType
     * @return The schema that includes the type definition
     */
    public static XmlElement getSchema(WsdlDefinitions definitions,
            QName paramType) {
        XmlElement types = definitions.getTypes();
        for (XmlElement schema : types.elements(WSConstants.XSD_NS,
                WSConstants.SCHEMA_TAG)) {
            String schemaTargetNamespace = schema
                    .attributeValue(WSConstants.TARGET_NAMESPACE_ATTRIBUTE);
            if (schemaTargetNamespace.equals(paramType.getNamespaceURI())) {
                for (XmlElement complexType : schema.elements(
                        WSConstants.XSD_NS, WSConstants.COMPLEX_TYPE_TAG)) {
                    String complexTypeName = complexType
                            .attributeValue(WSConstants.NAME_ATTRIBUTE);
                    if (complexTypeName.equals(paramType.getLocalPart())) {
                        return schema;
                    }
                }
                for (XmlElement simpleType : schema.elements(
                        WSConstants.XSD_NS, WSConstants.SIMPLE_TYPE_TAG)) {
                    String simpleTypeName = simpleType
                            .attributeValue(WSConstants.NAME_ATTRIBUTE);
                    if (simpleTypeName.equals(paramType.getLocalPart())) {
                        return schema;
                    }
                }
            }
        }
        return null;
    }

    /**
     * @param definitions
     * @param paramType
     * @return The type definition
     */
    public static XmlElement getTypeDefinition(WsdlDefinitions definitions,
            QName paramType) {
        XmlElement types = definitions.getTypes();
        for (XmlElement schema : types.elements(WSConstants.XSD_NS,
                WSConstants.SCHEMA_TAG)) {
            String schemaTargetNamespace = schema
                    .attributeValue(WSConstants.TARGET_NAMESPACE_ATTRIBUTE);
            if (schemaTargetNamespace.equals(paramType.getNamespaceURI())) {
                for (XmlElement complexType : schema.elements(
                        WSConstants.XSD_NS, WSConstants.COMPLEX_TYPE_TAG)) {
                    String complexTypeName = complexType
                            .attributeValue(WSConstants.NAME_ATTRIBUTE);
                    if (complexTypeName.equals(paramType.getLocalPart())) {
                        return complexType;
                    }
                }
                for (XmlElement simpleType : schema.elements(
                        WSConstants.XSD_NS, WSConstants.SIMPLE_TYPE_TAG)) {
                    String simpleTypeName = simpleType
                            .attributeValue(WSConstants.NAME_ATTRIBUTE);
                    if (simpleTypeName.equals(paramType.getLocalPart())) {
                        return simpleType;
                    }
                }
            }
        }
        return null;
    }

    /**
     * @param wsdl
     * @return true if the WSDL is AWSDL; false otherwise.
     */
    public static boolean isAWSDL(WsdlDefinitions wsdl) {
        if (wsdl.services().iterator().hasNext()) {
            return false;
        }
        return true;
    }

    /**
     * @param definitions
     * @return true if the service supports asynchronous invocation; false
     *         otherwise;
     */
    public static boolean isAsynchronousSupported(WsdlDefinitions definitions) {
        for (WsdlBinding binding : definitions.bindings()) {
            XmlElement element = binding.xml().element(
                    WSConstants.USING_ADDRESSING_TAG);
            if (element != null) {
                return true;
            }
        }
        return false;
    }

    /**
     * Converts a specified AWSDL to CWSDL using DSC URI.
     * 
     * @param definitions
     *            The specified AWSDL. This will be modified.
     * @param url
     *            The URL of the service
     * @return The CWSDL converted.
     */
    public static WsdlDefinitions convertToCWSDL(WsdlDefinitions definitions,
            URI url) {
        for (WsdlPortType portType : definitions.portTypes()) {
            WsdlUtil.createCWSDL(definitions, portType, url);
        }
        return definitions;
    }

    /**
     * @param uri
     * @return The URI with "?wsdl" at the end.
     */
    public static String appendWSDLQuary(String uri) {
        URI wsdlURI = appendWSDLQuary(URI.create(uri));
        return wsdlURI.toString();
    }

    /**
     * @param uri
     * @return The URI with "?wsdl" at the end.
     */
    public static URI appendWSDLQuary(URI uri) {
        if (uri.toString().endsWith("?wsdl")) {
            logger.warning("URL already has ?wsdl at the end: "
                    + uri.toString());
            // Don't throw exception to be more error tolerant.
            return uri;
        }
        String path = uri.getPath();
        if (path == null || path.length() == 0) {
            uri = uri.resolve("/");
        }
        uri = URI.create(uri.toString() + "?wsdl");
        return uri;
    }
}

/*
 * Indiana University Extreme! Lab Software License, Version 1.2
 * 
 * Copyright (c) 2006-2007 The Trustees of Indiana University. All rights
 * reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 * 1) All redistributions of source code must retain the above copyright notice,
 * the list of authors in the original source code, this list of conditions and
 * the disclaimer listed in this license;
 * 
 * 2) All redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the disclaimer listed in this license in
 * the documentation and/or other materials provided with the distribution;
 * 
 * 3) Any documentation included with all redistributions must include the
 * following acknowledgement:
 * 
 * "This product includes software developed by the Indiana University Extreme!
 * Lab. For further information please visit http://www.extreme.indiana.edu/"
 * 
 * Alternatively, this acknowledgment may appear in the software itself, and
 * wherever such third-party acknowledgments normally appear.
 * 
 * 4) The name "Indiana University" or "Indiana University Extreme! Lab" shall
 * not be used to endorse or promote products derived from this software without
 * prior written permission from Indiana University. For written permission,
 * please contact http://www.extreme.indiana.edu/.
 * 
 * 5) Products derived from this software may not use "Indiana University" name
 * nor may "Indiana University" appear in their name, without prior written
 * permission of the Indiana University.
 * 
 * Indiana University provides no reassurances that the source code provided
 * does not infringe the patent or any other intellectual property rights of any
 * other entity. Indiana University disclaims any liability to any recipient for
 * claims brought by any other entity based on infringement of intellectual
 * property rights or otherwise.
 * 
 * LICENSEE UNDERSTANDS THAT SOFTWARE IS PROVIDED "AS IS" FOR WHICH NO
 * WARRANTIES AS TO CAPABILITIES OR ACCURACY ARE MADE. INDIANA UNIVERSITY GIVES
 * NO WARRANTIES AND MAKES NO REPRESENTATION THAT SOFTWARE IS FREE OF
 * INFRINGEMENT OF THIRD PARTY PATENT, COPYRIGHT, OR OTHER PROPRIETARY RIGHTS.
 * INDIANA UNIVERSITY MAKES NO WARRANTIES THAT SOFTWARE IS FREE FROM "BUGS",
 * "VIRUSES", "TROJAN HORSES", "TRAP DOORS", "WORMS", OR OTHER HARMFUL CODE.
 * LICENSEE ASSUMES THE ENTIRE RISK AS TO THE PERFORMANCE OF SOFTWARE AND/OR
 * ASSOCIATED MATERIALS, AND TO THE PERFORMANCE AND VALIDITY OF INFORMATION
 * GENERATED USING SOFTWARE.
 */
